#include <stdio.h>
#include <unistd.h>
#include <wiringSerial.h>
#include "request.h"
#include "arm.h"

void arm(int mode){
    
    /*
    100の位が0ならエレベーターの外のボタンを押す、1なら中を押す
    10の位は今いる階
    1の位は行先の階
    */

    if(mode / 100 == 0){//エレベーターの外
        if(mode / 10 == 1){//今いる階が1階
            osu(1);//up
        }else if(mode / 10 == 4){//今いる階が4階
            osu(2);//middle
        }else{//今いる階が1,4階以外
            osu(3);//down
        }
    }else{//エレベーターの中
        osu(mode % 10);
    }
}


//arm用のArduinoとの通信の関数
static int fd_a;

int arduino_arm_open(){
	//fd = serialOpen("/dev/ttyACM0", 9600); //USB
	fd_a = serialOpen("/dev/ttyUSB0", 115200); //USB
	//fd = serialOpen("/dev/ttyAMA0", 9600); //GPIO
	
	if(fd_a < 0){
		//printf("failed to open Arduino_arm\n");
		//return -1;
		//0が無理だったら1を開く
		fd_a = serialOpen("/dev/ttyUSB1", 115200);
		if(fd_a < 0){
			printf("failed to open arm1\n");
			return -1;
			
		}else{
			// バッファのクリア
			while(serialDataAvail(fd_a) > 0){
				serialGetchar(fd_a);
			}
			
			printf("Arduino opened0\n");
			return 0;
		}
	}else{
		// バッファのクリア
		while(serialDataAvail(fd_a) > 0){
			serialGetchar(fd_a);
		}
		
		printf("Arduino_arm opened\n");
		return 0;
	}
}

void arduino_arm_close(){
	serialClose(fd_a);
	printf("Arduino_arm closed\n");
}

void arduino_arm_clear(){
	while(serialDataAvail(fd_a) > 0){
		serialGetchar(fd_a);
	}
}

int arduino_arm_read(serial_data_arm *serial_data) {
	int i;
	
	while(serialDataAvail(fd_a) >= 7){
		serial_data->val[0] = serialGetchar(fd_a);
		
		if(serial_data->val[0] >= 0x80){
			for(i = 1; i < 7; i++){
				serial_data->val[i] = serialGetchar(fd_a);
			}
			return 0;
		}
	}
	
	return -1;
}

void arduino_arm_write(serial_data_arm serial_data) {
	int i;

	for(i = 0; i < 7; i++){
		serialPutchar(fd_a, serial_data.val[i]);
	}
}

int arduino_arm_receive(command_data_arm *command_data) {
	serial_data_arm serial_data;
	
	if(arduino_arm_read(&serial_data) == 0){
		*command_data = arduino_arm_decode2(arduino_arm_decode1(serial_data));
		return 0;
	}else{
		//printf("Tanaka\n");
		return -1;
	}
}

void arduino_arm_send(command_data_arm command_data){
	arduino_arm_write(arduino_arm_encode1(arduino_arm_encode2(command_data)));
}

middle_data_arm arduino_arm_decode1(serial_data_arm serial_data) {
	middle_data_arm middle_data;
	
	middle_data.val[0] = ((serial_data.val[0] << 1) & 0xfe) | ((serial_data.val[1] >> 6) & 0x01);
	middle_data.val[1] = ((serial_data.val[1] << 2) & 0xfc) | ((serial_data.val[2] >> 5) & 0x03);
	middle_data.val[2] = ((serial_data.val[2] << 3) & 0xf8) | ((serial_data.val[3] >> 4) & 0x07);
	middle_data.val[3] = ((serial_data.val[3] << 4) & 0xf0) | ((serial_data.val[4] >> 3) & 0x0f);
	middle_data.val[4] = ((serial_data.val[4] << 5) & 0xe0) | ((serial_data.val[5] >> 2) & 0x1f);
	middle_data.val[5] = ((serial_data.val[5] << 6) & 0xc0) | ((serial_data.val[6] >> 1) & 0x3f);
	
	return middle_data;
}

command_data_arm arduino_arm_decode2(middle_data_arm middle_data) {
	command_data_arm command_data;
	
	command_data.val[0] = ((signed short)middle_data.val[0] << 8) | middle_data.val[1];
	command_data.val[1] = ((signed short)middle_data.val[2] << 8) | middle_data.val[3];
	command_data.val[2] = ((signed short)middle_data.val[4] << 8) | middle_data.val[5];
	
	return command_data;
}

serial_data_arm arduino_arm_encode1(middle_data_arm middle_data) {
	serial_data_arm serial_data;
	
	serial_data.val[0] =                              0x80  | ((middle_data.val[0] >> 1) & 0x7f);
	serial_data.val[1] = ((middle_data.val[0] << 6) & 0x40) | ((middle_data.val[1] >> 2) & 0x3f);
	serial_data.val[2] = ((middle_data.val[1] << 5) & 0x60) | ((middle_data.val[2] >> 3) & 0x1f);
	serial_data.val[3] = ((middle_data.val[2] << 4) & 0x70) | ((middle_data.val[3] >> 4) & 0x0f);
	serial_data.val[4] = ((middle_data.val[3] << 3) & 0x78) | ((middle_data.val[4] >> 5) & 0x07);
	serial_data.val[5] = ((middle_data.val[4] << 2) & 0x7c) | ((middle_data.val[5] >> 6) & 0x03);
	serial_data.val[6] = ((middle_data.val[5] << 1) & 0x7e);
	
	return serial_data;
}

middle_data_arm arduino_arm_encode2(command_data_arm command_data) {
	middle_data_arm middle_data;
	
	middle_data.val[0] = (command_data.val[0] >> 8) & 0x00ff;
	middle_data.val[1] =  command_data.val[0]       & 0x00ff;
	middle_data.val[2] = (command_data.val[1] >> 8) & 0x00ff;
	middle_data.val[3] =  command_data.val[1]       & 0x00ff;
	middle_data.val[4] = (command_data.val[2] >> 8) & 0x00ff;
	middle_data.val[5] =  command_data.val[2]       & 0x00ff;
	
	return middle_data;
}

void osu(int mode){
	command_data_arm command_data;
	
	command_data.val[0] = 24;
	command_data.val[1] = mode;
	
	arduino_arm_clear();
	arduino_arm_send(command_data);
	
	usleep(50 * 1000);
	/*if(arduino_arm_receive(&command_data) == 0){
		if(command_data.val[1] == 100){
			printf("OK!!!\n");
		}else{
			printf("NG!!!\n");
		}
	}else{
		printf("error_osu\n");
	}*/
}

