#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <signal.h>
#include "pilot.h"
#include "arduino.h"
#include "arm.h"
#include "io.h"
#include "callback.h"
#include "input.h"
#include "status.h"

//Ctrl+Cで終了できるようにする
static volatile sig_atomic_t stop = 0;

void handle_signal(int signum) {
    if (signum == SIGINT) {
        stop = 1;
    }
}

//flagが0なら休み時間、1なら受業中
int time_judge(){
    // 現在の時刻を取得
    time_t t = time(NULL);
    struct tm *local_time = localtime(&t);

    int hour = local_time->tm_hour;
    int min = local_time->tm_min;
    int flag = 0;

    if(hour == 8 && min >= 50){
        flag = 1;
    }else if(hour == 9){
        flag = 1;
    }else if(hour == 10 && (min <= 20 || min >= 30)){
        flag = 1;
    }else if(hour == 11){
        flag = 1;
    }else if(hour == 13 && min >= 10){
        flag = 1;
    }else if(hour == 14 && (min <= 40 || min >= 50)){
        flag = 1;
    }else if(hour == 15){
        flag = 1;
    }else if(hour == 16 || min <= 20){
        flag = 1;
    }

    return flag;
}

int main(){
    int sw;

    if(arduino_arm_open() != 0){
        printf("armが正常にセットアップ出来ませんでした。\n");
        return -1;
    }
    if(io_open() != 0){
        printf("タッチセンサが正常にセットアップ出来ませんでした。\n");
        return -1;
    }
    if(callback_set() != 0){
        printf("強制停止ボタンが正常にセットアップ出来ませんでした。\n");
        return -1;
    }
    if(arduino_open() != 0){
        printf("Arduinoが正常にセットアップできませんでした。\n");
        return -1;
    }
    if(color_set() != 0){
        printf("カラーセンサが正常にセットアップ出来ませんでした。\n");
        return -1;
    }

    // 現在の時刻を取得
    time_t t = time(NULL);
    struct tm *local_time = localtime(&t);

    // ファイル名に現在の時刻を使用して生成
    char filename[100];
    sprintf(filename, "log/%04d%02d%02d_%02d%02d%02d.txt",
            local_time->tm_year + 1900, local_time->tm_mon + 1, local_time->tm_mday,
            local_time->tm_hour, local_time->tm_min, local_time->tm_sec);

    FILE *fp = fopen(filename, "a");
    if (fp == NULL) {
        perror("fopen");
        return -1;
    }
    fclose(fp);

    signal(SIGINT, handle_signal);


    sleep(1);

    int pet_state = 0, cap_state = 0;
    int photo0, photo, photo_d;
    photo0 = 1000;
    int yowai_frag = 0;

    int cap_no = 0, pet_no = 0, bun_no = 0;
    int angry = 5;
    int capa = 0;
    char command[50];
    
    int i=0;

    usleep(2000 * 1000);

    while(!stop){
        cap_photo(&photo);
        photo_d = photo - photo0;
        printf(" photo : %d\n", photo);
        //printf(" photo_d : %d\n", photo_d);

        io_get_sw(&sw); // タッチセンサの値を取得
        if(sw == 0){ // タッチセンサが押されたらアームを動かす
            
            t = time(NULL);// 現在の時間を取得
            local_time = localtime(&t);// 地元の時間に変換
            fopen(filename, "a");
            fprintf(fp , "%02d:%02d:%02d : ", local_time->tm_hour, local_time->tm_min, local_time->tm_sec);// 時刻を出力
            fprintf(fp , "armを動かしました\n");
            fclose(fp);
            
            printf("armを動かします。\n");

            osu(10);
            sleep(3);
            osu(2);
            sleep(2);
        }

        if (photo == 1023){
            cap_state = 1;
            yowai_frag = 1;
        }
        else if(photo != 1023 && yowai_frag == 1){
            cap_state = 0;
            yowai_frag = 0;
        }

        pet_state = water_input(); //(lux > 500)

        if (pet_state == 0 && cap_state == 0){
            /*投入されるのを待っている時の処理*/
        }
        else if (pet_state == 1 && cap_state == 0){
            /*キャップをセットしてね*/
            
            cap_no++;

            t = time(NULL);// 現在の時間を取得
            local_time = localtime(&t);// 地元の時間に変換
            fopen(filename, "a");
            fprintf(fp , "%02d:%02d:%02d : ", local_time->tm_hour, local_time->tm_min, local_time->tm_sec);// 時刻を出力
            fprintf(fp , "%d回目 キャップをいれてください\n",cap_no);
            fclose(fp);

            if(time_judge() == 0){
                speak(cap_speak);
            }            
            
            if(cap_no >= angry){//規定回数を超えると怒る
                fopen(filename, "a");
                fprintf(fp , "%02d:%02d:%02d : ", local_time->tm_hour, local_time->tm_min, local_time->tm_sec);// 時刻を出力
                fprintf(fp , "ずんだもんキャップがなくて怒る\n");
                fclose(fp);
                if(time_judge() == 0){
                    sprintf(command,"aplay speak/goback.wav");
				    system(command);
                }
                cap_no = 0;
				sleep(1);
            }
        }
        else if (pet_state == 0 && cap_state == 1){
            /*ペットボトルを入れてね*/

            pet_no++;

            t = time(NULL);// 現在の時間を取得
            local_time = localtime(&t);// 地元の時間に変換
            fopen(filename, "a");
            fprintf(fp , "%02d:%02d:%02d : ", local_time->tm_hour, local_time->tm_min, local_time->tm_sec);// 時刻を出力
            fprintf(fp , "%d回目 ペットボトルを入れてください\n",pet_no);
            fclose(fp);

            if(time_judge() == 0){
                speak(accept_speak);
            }            

            if(pet_no >= angry){//規定回数を超えると怒る
                fopen(filename, "a");
                fprintf(fp , "%02d:%02d:%02d : ", local_time->tm_hour, local_time->tm_min, local_time->tm_sec);// 時刻を出力
                fprintf(fp , "ずんだもんペットボトルがなくて怒る\n");
                fclose(fp);
                if(time_judge() == 0){
                    sprintf(command,"aplay speak/goback.wav");
				    system(command);
                }     
                pet_no = 0;
				sleep(1);
            }
        }
        else if (pet_state == 1 && cap_state == 1){
            if (water() == 0){
                pass(); // モータを動かす
                led(gre_fla);
                led(gre_fla);
                /*リサイクルありがとう！*/
                speak(thank_speak);
                usleep(2000 * 1000);

                cap_no = 0;
                pet_no = 0;
                bun_no = 0;
                capa++;
                
                t = time(NULL);// 現在の時間を取得
                local_time = localtime(&t);// 地元の時間に変換
                fopen(filename, "a");
                fprintf(fp , "%02d:%02d:%02d : ", local_time->tm_hour, local_time->tm_min, local_time->tm_sec);// 時刻を出力
                fprintf(fp , "%d 個目のペットボトルが投入されました。\n",capa);
                fclose(fp);

                if(capa >= 27){
                    printf("容量がいっぱいです\n");
                    fopen(filename, "a");
                    fprintf(fp , "%02d:%02d:%02d : ", local_time->tm_hour, local_time->tm_min, local_time->tm_sec);// 時刻を出力
                    fprintf(fp , "容量がいっぱいです\n");
                    fclose(fp);
                    
                    sprintf(command,"aplay speak/full.wav");
                    system(command);
                    sleep(1);
                    while(1){
                        led(red_fla);
                        sleep(5);
                    }
                }

            }else{
                /*ペットボトルの分別ができていないよ*/
                speak(bunbetsu_speak);
                bun_no++;

                t = time(NULL);// 現在の時間を取得
                local_time = localtime(&t);// 地元の時間に変換
                fopen(filename, "a");
                fprintf(fp , "%02d:%02d:%02d : ", local_time->tm_hour, local_time->tm_min, local_time->tm_sec);// 時刻を出力
                fprintf(fp , "%d回目 分別をしてください\n",bun_no);
                fclose(fp);
                
                if(bun_no >= angry){//規定回数を超えると怒る
                    fopen(filename, "a");
                    fprintf(fp , "%02d:%02d:%02d : ", local_time->tm_hour, local_time->tm_min, local_time->tm_sec);// 時刻を出力
                    fprintf(fp , "ずんだもん分別ができてなくて怒る\n");
                    fclose(fp);

                    sprintf(command,"aplay speak/goback.wav");
                    system(command);
                    bun_no = 0;
                    sleep(1);
                }
            }
        }
        photo0 = photo;
        
        if(i == 60){//定期的にArduinoを開き直す
            i = 0;
            if(arduino_open() != 0){
                printf("Arduinoが正常にセットアップできませんでした。\n");
                while(1){
                    led(red_fla);
                    sleep(5);
                }
            }
            if(arduino_arm_open() != 0){
                printf("Arduinoが正常にセットアップできませんでした。\n");
                while(1){
                    led(red_fla);
                    sleep(5);
                }
            }
            do{
                cap_photo(&photo);
                led(red_fla);
                sleep(1);
            }while(photo > 1023);
        }
        i++;
        usleep(500 * 1000);
    }

    fclose(fp);
    arduino_close();
    arduino_arm_close();
    return 0;
}




