#include <stdio.h>
#include <unistd.h>
#include <wiringPiI2C.h>
#include "uss.h"
#define ADDRESS_MOTER_L 0x70
#define ADDRESS_MOTER_R 0x73
#define ADDRESS_USS_PET 0x75
#define ADDRESS_USS_MIGI 0x72
#define ADDRESS_USS_HIDARI 0x75

// 測定の時間間隔[ms]
static const int t_uss = 50;

// 測定範囲の下限・上限[cm]
static const int uss_min =  16;
static const int uss_max = 600;

// センサからMIRS中心までの距離[cm]
static const int dist_center = -10;

static int fd_l, fd_r;

/*ID : i2cdetect -y 1 */

int uss_open_l(){
	fd_l = wiringPiI2CSetup(ADDRESS_MOTER_L);//ID_Left
	
	// ソフトウェアリビジョンの確認
	if(wiringPiI2CReadReg8(fd_l, 0x00) == 0x06){
		printf("USS_L opened\n");
		return 0;
	}else{
		printf("failed to open USS_L\n");
		return -1;
	}
}

int uss_open_r(){
	fd_r = wiringPiI2CSetup(ADDRESS_MOTER_R);//ID_Right
	
	// ソフトウェアリビジョンの確認
	if(wiringPiI2CReadReg8(fd_r, 0x00) == 0x06){
		printf("USS_R opened\n");
		return 0;
	}else{
		printf("failed to open USS_R\n");
		return -1;
	}
}

long uss_get_l(){
	unsigned char lowbyte, highbyte;
	long val;
	
	wiringPiI2CWriteReg8(fd_l, 0x00, 0x51);
	usleep(t_uss * 1000);
	
	lowbyte  = wiringPiI2CReadReg8(fd_l, 0x03);
	highbyte = wiringPiI2CReadReg8(fd_l, 0x02);
	val = lowbyte + highbyte * 256;
	
	if(val < uss_min || val > uss_max){
		return -1;
	}else{
		return val + dist_center;
	}
}

long uss_get_r(){
	unsigned char lowbyte, highbyte;
	long val;
	
	wiringPiI2CWriteReg8(fd_r, 0x00, 0x51);
	usleep(t_uss * 1000);
	
	lowbyte  = wiringPiI2CReadReg8(fd_r, 0x03);
	highbyte = wiringPiI2CReadReg8(fd_r, 0x02);
	val = lowbyte + highbyte * 256;
	
	if(val < uss_min || val > uss_max){
		return -1;
	}else{
		return val + dist_center;
	}
}






/*ここからボトル用プログラム*/

// 測定範囲の下限・上限[cm]
static const int uss_pet_min =  5;
static const int uss_pet_max = 600;

static int fd_pet;

int uss_open_pet(){
	fd_pet = wiringPiI2CSetup(ADDRESS_USS_PET);//ID_pet
	
	// ソフトウェアリビジョンの確認
	if(wiringPiI2CReadReg8(fd_pet, 0x00) == 0x06){
		printf("USS_pet opened\n");
		return 0;
	}else{
		printf("failed to open USS_pet\n");
		return -1;
	}
}

long uss_get_pet(){
	unsigned char lowbyte, highbyte;
	long val;
	
	wiringPiI2CWriteReg8(fd_pet, 0x00, 0x51);
	usleep(t_uss * 1000);
	
	lowbyte  = wiringPiI2CReadReg8(fd_pet, 0x03);
	highbyte = wiringPiI2CReadReg8(fd_pet, 0x02);
	val = lowbyte + highbyte * 256;
	
	if(val < uss_pet_min || val > uss_pet_max){
		return -1;
	}else{
		return val;
	}
}


//ここから走行用(右)

static int fd_migi;

int uss_open_migi(){
	fd_migi = wiringPiI2CSetup(ADDRESS_USS_MIGI);//ID_cap
	
	// ソフトウェアリビジョンの確認
	if(wiringPiI2CReadReg8(fd_migi, 0x00) == 0x06){
		printf("USS_migi opened\n");
		return 0;
	}else{
		printf("failed to open USS_migi\n");
		return -1;
	}
}

long uss_get_migi(){
	unsigned char lowbyte, highbyte;
	long val;
	
	wiringPiI2CWriteReg8(fd_migi, 0x00, 0x51);
	usleep(t_uss * 1000);
	
	lowbyte  = wiringPiI2CReadReg8(fd_migi, 0x03);
	highbyte = wiringPiI2CReadReg8(fd_migi, 0x02);
	val = lowbyte + highbyte * 256;
	
	if(val < uss_min || val > uss_max){
		return -1;
	}else{
		return val;
	}
}

//ここから走行用(左)

static int fd_hidari;

int uss_open_hidari(){
	fd_hidari = wiringPiI2CSetup(ADDRESS_USS_HIDARI);//ID_cap
	
	// ソフトウェアリビジョンの確認
	if(wiringPiI2CReadReg8(fd_hidari, 0x00) == 0x06){
		printf("USS_hidari opened\n");
		return 0;
	}else{
		printf("failed to open USS_hidari\n");
		return -1;
	}
}

long uss_get_hidari(){
	unsigned char lowbyte, highbyte;
	long val;
	
	wiringPiI2CWriteReg8(fd_hidari, 0x00, 0x51);
	usleep(t_uss * 1000);
	
	lowbyte  = wiringPiI2CReadReg8(fd_hidari, 0x03);
	highbyte = wiringPiI2CReadReg8(fd_hidari, 0x02);
	val = lowbyte + highbyte * 256;
	
	if(val < uss_min || val > uss_max){
		return -1;
	}else{
		return val;
	}
}

